import { act, render, screen, waitFor } from '@testing-library/react'
import { userEvent } from '@testing-library/user-event'
import { PerformanceIndicator } from 'models'
import React from 'react'
import { Provider } from 'react-redux'
import {
  mockDataLoadEnedisAnalysis,
  mockEnedisMonthlyAnalysisArray,
} from 'tests/__mocks__/enedisMonthlyAnalysisData.mock'
import { allLastFluidPrices } from 'tests/__mocks__/fluidPrice.mock'
import { createMockEcolyoStore } from 'tests/__mocks__/store'
import ElecHalfHourMonthlyAnalysis from './ElecHalfHourMonthlyAnalysis'

jest.mock('components/Hooks/useExploration', () => {
  return () => ['', jest.fn()]
})

jest.mock(
  'components/Analysis/ElecHalfHourMonthlyAnalysis/ElecHalfHourChart',
  () => 'mock-elechalfhourchart'
)

const mockCheckDoctypeEntries = jest.fn()
jest.mock('services/consumption.service', () => {
  return jest.fn(() => ({
    checkDoctypeEntries: mockCheckDoctypeEntries,
  }))
})
const mockGetEnedisMonthlyAnalysisByDate = jest.fn()
const mockAggregateValuesToDataLoad = jest.fn()
jest.mock('services/enedisMonthlyAnalysisData.service', () => {
  return jest.fn(() => ({
    getEnedisMonthlyAnalysisByDate: mockGetEnedisMonthlyAnalysisByDate,
    aggregateValuesToDataLoad: mockAggregateValuesToDataLoad,
    getOffPeakHours: () => null,
  }))
})
const mockPerfIndicator: PerformanceIndicator = {
  value: 10,
  compareValue: null,
  percentageVariation: null,
  price: null,
}
const mockGetPrices = jest.fn()
jest.mock('services/fluidsPrices.service', () => {
  return jest.fn(() => ({
    getPrices: mockGetPrices,
  }))
})

describe('ElecHalfHourMonthlyAnalysis component', () => {
  const store = createMockEcolyoStore()
  beforeEach(() => {
    jest.clearAllMocks()
  })
  it('should be rendered correctly when isHalfHourActivated is false', async () => {
    mockCheckDoctypeEntries.mockResolvedValueOnce(false)
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])

    const { container } = render(
      <Provider store={store}>
        <ElecHalfHourMonthlyAnalysis perfIndicator={mockPerfIndicator} />
      </Provider>
    )
    await waitFor(() => null, { container })
    expect(container).toMatchSnapshot()
  })

  it('should be rendered correctly when isHalfHourActivated is true and render weekend graph', async () => {
    mockCheckDoctypeEntries.mockResolvedValue(true)
    mockGetEnedisMonthlyAnalysisByDate.mockResolvedValueOnce(
      mockEnedisMonthlyAnalysisArray
    )
    mockAggregateValuesToDataLoad.mockResolvedValueOnce(
      mockDataLoadEnedisAnalysis
    )
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])

    const { container } = render(
      <Provider store={store}>
        <ElecHalfHourMonthlyAnalysis perfIndicator={mockPerfIndicator} />
      </Provider>
    )
    await waitFor(() => null, { container })
    expect(container).toMatchSnapshot()
    expect(screen.getByText('special_elec.weekend')).toBeInTheDocument()
  })

  it('should change from weekend to week', async () => {
    mockCheckDoctypeEntries.mockResolvedValue(true)
    mockGetEnedisMonthlyAnalysisByDate.mockResolvedValueOnce(
      mockEnedisMonthlyAnalysisArray
    )
    mockAggregateValuesToDataLoad.mockResolvedValueOnce(
      mockDataLoadEnedisAnalysis
    )
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])
    render(
      <Provider store={store}>
        <ElecHalfHourMonthlyAnalysis perfIndicator={mockPerfIndicator} />
      </Provider>
    )
    await screen.findByLabelText(
      'consumption.accessibility.button_previous_value'
    )
    await act(async () => {
      await userEvent.click(
        screen.getByLabelText('consumption.accessibility.button_previous_value')
      )
    })
    expect(screen.getByText('special_elec.week')).toBeInTheDocument()
  })

  it('should call the ElecInfoModal with open = true when click on the button', async () => {
    mockCheckDoctypeEntries.mockResolvedValue(true)
    mockGetEnedisMonthlyAnalysisByDate.mockResolvedValueOnce(
      mockEnedisMonthlyAnalysisArray
    )
    mockAggregateValuesToDataLoad.mockReturnValueOnce(
      mockDataLoadEnedisAnalysis
    )
    mockGetPrices.mockResolvedValue(allLastFluidPrices[0])
    render(
      <Provider store={store}>
        <ElecHalfHourMonthlyAnalysis perfIndicator={mockPerfIndicator} />
      </Provider>
    )

    await waitFor(async () => {
      await userEvent.click(screen.getByText('special_elec.showModal'))
    })
    expect(await screen.findByRole('dialog')).toBeInTheDocument()
  })
})
