import { act, render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import { useI18n } from 'cozy-ui/transpiled/react/providers/I18n'
import React from 'react'
import { Provider } from 'react-redux'
import { createMockEcolyoStore } from 'tests/__mocks__/store'
import '../../../locales/fr.json'
import EcogestureInitModal from './EcogestureInitModal'

const mockedNavigate = jest.fn()
const mockOnClose = jest.fn()
const mockOnAccept = jest.fn()

jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

describe('EcogestureInitModal component', () => {
  const store = createMockEcolyoStore()
  const { t } = useI18n()

  beforeEach(() => {
    jest.clearAllMocks()
  })

  it('should not render the modal when open is false', () => {
    render(
      <Provider store={store}>
        <EcogestureInitModal
          open={false}
          onClose={mockOnClose}
          onAccept={mockOnAccept}
        />
      </Provider>
    )
    expect(screen.queryByRole('dialog')).toBeNull()
  })

  it('should render correctly when open is true', () => {
    const { baseElement } = render(
      <Provider store={store}>
        <EcogestureInitModal
          open={true}
          onClose={mockOnClose}
          onAccept={mockOnAccept}
        />
      </Provider>
    )
    expect(baseElement).toMatchSnapshot()
  })

  it('should call onAccept and navigate to /ecogesture-form?modal=false when the accept button is clicked', async () => {
    render(
      <Provider store={store}>
        <EcogestureInitModal
          open={true}
          onClose={mockOnClose}
          onAccept={() => {
            mockOnAccept()
            mockedNavigate('/ecogesture-form?modal=false')
          }}
        />
      </Provider>
    )
    await act(async () => {
      await userEvent.click(screen.getByText(t('ecogesture.initModal.btn2')))
    })
    expect(mockedNavigate).toHaveBeenCalledWith('/ecogesture-form?modal=false')
    expect(mockOnAccept).toHaveBeenCalled()
  })

  it('should call onClose and navigate to /ecogestures when the close button is clicked', async () => {
    render(
      <Provider store={store}>
        <EcogestureInitModal
          open={true}
          onClose={() => {
            mockOnClose()
            mockedNavigate('/ecogestures')
          }}
          onAccept={mockOnAccept}
        />
      </Provider>
    )
    await act(async () => {
      await userEvent.click(screen.getByText(t('ecogesture.initModal.btn1')))
    })
    expect(mockedNavigate).toHaveBeenCalledWith('/ecogestures')
    expect(mockOnClose).toHaveBeenCalled()
  })
})
