import { QueryResult } from 'cozy-client'
import { UserExplorationState } from 'enums'
import { ExplorationEntity, UserChallenge, UserExploration } from 'models'
import mockClient from 'tests/__mocks__/client.mock'
import {
  allExplorationEntities,
  explorationEntity,
  userExplorationDone,
  userExplorationStarted,
  userExplorationUnlocked,
} from 'tests/__mocks__/explorationData.mock'
import { getError } from 'tests/__mocks__/testUtils'
import {
  userChallengeExplo1OnGoing,
  userChallengeExplo2OnGoing,
} from 'tests/__mocks__/userChallengeData.mock'
import ExplorationService from './exploration.service'

describe('Exploration service', () => {
  const explorationService = new ExplorationService(mockClient)

  describe('getAllExplorationEntities method', () => {
    it('should return all exploration entity', async () => {
      const mockQueryResult: QueryResult<ExplorationEntity[]> = {
        data: allExplorationEntities,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result = await explorationService.getAllExplorationEntities()
      expect(result).toEqual(allExplorationEntities)
    })
  })

  describe('getExplorationEntityById method', () => {
    it('should return one exploration by id', async () => {
      const mockQueryResult: QueryResult<ExplorationEntity[]> = {
        data: [explorationEntity],
        bookmark: '',
        next: false,
        skip: 0,
      }
      const explorationId = 'EXPLORATION001'
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      const result =
        await explorationService.getExplorationEntityById(explorationId)
      expect(result).toEqual(explorationEntity)
    })
  })

  describe('deleteAllExplorationEntities method', () => {
    it('should delete all exploration entities', async () => {
      const mockQueryResult: QueryResult<ExplorationEntity[]> = {
        data: allExplorationEntities,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)

      const result = await explorationService.deleteAllExplorationEntities()
      expect(result).toBeTruthy()
    })

    it('should throw an error because destroy failed', async () => {
      const mockQueryResult: QueryResult<ExplorationEntity[]> = {
        data: allExplorationEntities,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.query.mockResolvedValueOnce(mockQueryResult)
      mockClient.destroy.mockRejectedValue(new Error())
      const error = await getError(async () =>
        explorationService.deleteAllExplorationEntities()
      )
      expect(error).toEqual(new Error())
    })
  })

  describe('parseExplorationEntityToUserExploration method', () => {
    it('should return the userExploration from a explorationEntity', () => {
      const result =
        explorationService.parseExplorationEntityToUserExploration(
          explorationEntity
        )
      const mockUpdatedExploration: UserExploration = {
        ...explorationEntity,
        progress: 0,
        date: result.date,
      }
      expect(result).toEqual(mockUpdatedExploration)
    })
  })

  describe('getUserExplorationFromExplorationEntities method', () => {
    it('should return the userExploration from a explorationEntity', () => {
      const searchId = 'EXPLORATION001'
      const result =
        explorationService.getUserExplorationFromExplorationEntities(
          allExplorationEntities,
          searchId
        )
      const updatedUserExploration = {
        ...userExplorationUnlocked,
        date: result.date,
      }
      expect(result).toEqual(updatedUserExploration)
    })
  })
  describe('startUserExploration Method', () => {
    it('should return the started userExploration', () => {
      const result = explorationService.startUserExploration(
        userExplorationUnlocked
      )
      expect(result).toEqual(userExplorationStarted)
    })
  })
  describe('endUserExploration Method', () => {
    it('should return the finished userExploration', () => {
      const result = explorationService.endUserExploration(
        userExplorationStarted
      )
      expect(result).toEqual(userExplorationDone)
    })
  })
  describe('checkExploration Method', () => {
    it('should set the exploration to done', async () => {
      const updatedExploration: UserExploration = {
        ...userChallengeExplo2OnGoing.exploration,
        progress: 1,
        state: UserExplorationState.DONE,
      }
      const updatedChallengeExploDone = {
        ...userChallengeExplo2OnGoing,
        exploration: updatedExploration,
      }
      const mockQueryResult: QueryResult<UserChallenge> = {
        data: updatedChallengeExploDone,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockResolvedValueOnce(mockQueryResult)
      const result = await explorationService.checkExploration(
        userChallengeExplo2OnGoing,
        'EXPLORATION002'
      )

      expect(result).toEqual(updatedChallengeExploDone)
    })
    it('should update the exploration progress', async () => {
      const updatedExploration: UserExploration = {
        ...userChallengeExplo2OnGoing.exploration,
        progress: 2,
      }
      const updatedChallengeExploDone = {
        ...userChallengeExplo2OnGoing,
        exploration: updatedExploration,
      }
      const mockQueryResult: QueryResult<UserChallenge> = {
        data: updatedChallengeExploDone,
        bookmark: '',
        next: false,
        skip: 0,
      }
      mockClient.save.mockResolvedValueOnce(mockQueryResult)
      const result = await explorationService.checkExploration(
        userChallengeExplo1OnGoing,
        'EXPLORATION001'
      )
      expect(result).toEqual(updatedChallengeExploDone)
    })
  })
})
